import json
import logging
import re
from typing import Optional

from daktari.check import Check, CheckResult
from daktari.command_utils import get_stdout
from daktari.os import OS


class KubectlInstalled(Check):
    def __init__(self, minimum_version: Optional[float] = None):
        self.minimum_version = minimum_version
        self.name = "kubectl.installed"
        self.suggestions = {
            OS.OS_X: "<cmd>brew install kubectl</cmd>",
            OS.UBUNTU: "<cmd>sudo snap install kubectl --classic</cmd>",
            OS.GENERIC: "Install kubectl: https://kubernetes.io/docs/tasks/tools/#kubectl",
        }

    def check(self) -> CheckResult:
        installed_version = get_kubectl_version()
        return self.validate_minimum_version("Kubectl", installed_version, self.minimum_version)


version_pattern = re.compile("Client Version: v([0-9]+.[0-9]+)")


def get_kubectl_version() -> Optional[float]:
    raw_version = get_stdout("kubectl version --client=true --short")
    if raw_version:
        match = version_pattern.search(raw_version)
        if match:
            version_string = match.group(1)
            logging.debug(f"Kubectl version: {version_string}")
            return float(version_string)
    return None


class KubectlContextExists(Check):
    def __init__(self, context_name: str, provision_command: str = ""):
        self.context_name = context_name
        self.name = f"kubectl.contextExists.{context_name}"
        self.suggestions = {OS.GENERIC: provision_command}

    def check(self) -> CheckResult:
        output = get_stdout("kubectl config get-contexts")
        passed = bool(output and self.context_name in output)
        return self.verify(passed, f"{self.context_name} is <not/> configured for the current user")


class HelmInstalled(Check):
    name = "helm.installed"

    def __init__(self, minimum_version: Optional[float] = None):
        self.minimum_version = minimum_version
        self.suggestions = {
            OS.OS_X: "<cmd>brew install helm</cmd>",
            OS.UBUNTU: "<cmd>sudo snap install helm --classic</cmd>",
            OS.GENERIC: "Install Helm: https://helm.sh/docs/intro/install/",
        }

    def check(self) -> CheckResult:
        installed_version = get_helm_version()
        return self.validate_minimum_version("Helm", installed_version, self.minimum_version)


helm_version_pattern = re.compile("v([0-9]+.[0-9]+)")


def get_helm_version() -> Optional[float]:
    raw_version = get_stdout("helm version --short")
    if raw_version:
        match = helm_version_pattern.search(raw_version)
        if match:
            version_string = match.group(1)
            logging.debug(f"Helm Version: {version_string}")
            return float(version_string)
    return None


class HelmRepoExists(Check):
    depends_on = [HelmInstalled]

    def __init__(self, repo_name: str, repo_url: str):
        self.repo_name = repo_name
        self.repo_url = repo_url.strip("/")
        self.name = f"helm.repoExists.{repo_name}"
        self.suggestions = {
            OS.GENERIC: f"<cmd>helm repo add {repo_name} {repo_url} --force-update</cmd>",
        }

    def check(self) -> CheckResult:
        output = get_stdout("helm repo list -o json")
        if not output:
            return self.failed("No helm repos appear to be configured for the current user.")
        repo_json = json.loads(output)
        repo = next(filter(lambda repo_details: repo_details.get("name") == self.repo_name, repo_json), None)
        if repo is None:
            return self.failed(f"{self.repo_name} is not configured for the current user")

        installed_url = repo["url"].strip("/")
        logging.debug(f"{self.repo_name} helm repo is installed with URL {installed_url}.")

        if installed_url != self.repo_url:
            return self.failed(
                f"{self.repo_name} is configured to use the wrong URL. Expected {self.repo_url}, got {installed_url}"
            )

        return self.passed(f"{self.repo_name} is configured for the current user")



import logging
import re
from typing import Optional

from daktari.check import Check, CheckResult
from daktari.command_utils import get_stdout
from daktari.os import OS


class TfenvInstalled(Check):
    name = "tfenv.installed"

    suggestions = {
        OS.OS_X: "<cmd>brew install tfenv</cmd>",
        OS.UBUNTU: """<cmd>git clone https://github.com/tfutils/tfenv.git ~/.tfenv</cmd>
                      <cmd>sudo ln -s ~/.tfenv/bin/* /usr/local/bin</cmd>""",
        OS.GENERIC: "Install tfenv: https://github.com/tfutils/tfenv",
    }

    def check(self) -> CheckResult:
        return self.verify_install("tfenv")


class TerraformInstalled(Check):
    def __init__(self, required_version: str = "", use_tfenv: bool = False):
        self.required_version = required_version
        self.name = "terraform.installed"
        self.use_tfenv = use_tfenv

        if use_tfenv:
            self.depends_on = [TfenvInstalled]

            # Read the required tf-version from tfenv (i.e. .terraform-version files)
            tfenv_version = open(".terraform-version", "r").read().strip()
            logging.debug(f"Terraform version required from .terraform-version file is: {tfenv_version}")
            self.required_version = tfenv_version
            self.suggestions = {OS.GENERIC: "<cmd>tfenv install</cmd>"}
        else:
            version_string = f"@{required_version}" if required_version else ""
            self.suggestions = {
                OS.OS_X: f"<cmd>brew tap hashicorp/tap && hashicorp/tap/terraform{version_string}</cmd>",
                OS.GENERIC: "Install Terraform: https://learn.hashicorp.com/tutorials/terraform/install-cli",
            }

    def check(self) -> CheckResult:
        installed_version = get_terraform_version()
        return self.validate_required_version(
            "Terraform", installed_version=installed_version, required_version=self.required_version
        )


version_pattern = re.compile("Terraform v([0-9]+.[0-9]+.[0-9]+)")


def get_terraform_version() -> Optional[str]:
    raw_version = get_stdout("terraform version")
    if raw_version:
        match = version_pattern.search(raw_version)
        if match:
            version_string = match.group(1)
            logging.debug(f"Terraform version: {version_string}")
            return str(version_string)
    return None



from os.path import expanduser

from daktari.check import Check, CheckResult
from daktari.file_utils import file_exists
from daktari.os import OS


class FileExists(Check):
    name = "file.exists"

    def __init__(self, file_path: str, suggestion: str):
        self.file_path = file_path.replace("~", expanduser("~"))
        self.suggestions = {OS.GENERIC: suggestion}

    def check(self) -> CheckResult:
        return self.verify(file_exists(self.file_path), f"{self.file_path} is <not/> present")



import json
import logging
from pathlib import Path
from semver import VersionInfo
from typing import Optional

from daktari.check import Check, CheckResult
from daktari.command_utils import get_stdout
from daktari.file_utils import file_exists
from daktari.os import OS


class OnePassInstalled(Check):
    name = "onepass.installed"

    def __init__(self, required_version: Optional[str] = None, recommended_version: Optional[str] = None):
        self.required_version = required_version
        self.recommended_version = recommended_version
        self.suggestions = {
            OS.GENERIC: "Install OP (1Password CLI): "
            "https://support.1password.com/command-line-getting-started/#set-up-the-command-line-tool",
        }

    def check(self) -> CheckResult:
        installed_version = get_op_version()
        return self.validate_semver_expression(
            "1Password CLI", installed_version, self.required_version, self.recommended_version
        )


def get_op_version() -> Optional[float]:
    raw_version = get_stdout("op --version")
    if raw_version:
        try:
            version = VersionInfo.parse(raw_version)
        except ValueError:
            return None
        logging.debug(f"OP Version: {version}")
        return version
    return None


class OPAccountExists(Check):
    depends_on = [OnePassInstalled]
    name = "onepass.contextExists"

    def __init__(self, context_name: str):
        self.context_name = context_name
        self.suggestions = {
            OS.GENERIC: f"<cmd>op signin {context_name}.1password.com <your-email-here></cmd>",
        }

    def check(self) -> CheckResult:

        home = str(Path.home())
        possible_paths = [f"{home}/.op/config", f"{home}/.config/op/config"]

        for config_path in possible_paths:
            if file_exists(config_path):
                op_config = json.loads(open(config_path).read())
                account_json = op_config["accounts"]
                repo = next(
                    filter(lambda op_contexts: op_contexts.get("shorthand") == self.context_name, account_json), None
                )
                if repo is None:
                    return self.failed(f"{self.context_name} is not configured with OP CLI for the current user")

                return self.passed(f"{self.context_name} is configured with OP CLI for the current user")

        return self.failed("No 1Password config appears to be present on this machine.")



import logging
import re
from dataclasses import dataclass
from pathlib import Path
from typing import Any, Dict, List, Optional

from colors import red, yellow
from packaging import version

from daktari import __version__
from daktari.check import Check
from daktari.result_printer import print_suggestion_text


@dataclass
class Config:
    min_version: Optional[str]
    title: Optional[str]
    checks: List[Check]


version_regex = re.compile('daktari_version.*"([0-9.]+)"')


def read_config(config_path: Path) -> Optional[Config]:
    raw_config = config_path.read_text()
    return parse_raw_config(config_path, raw_config)


def parse_raw_config(config_path: Path, raw_config: str) -> Optional[Config]:
    if not check_version_compatibility(config_path, raw_config):
        return None

    variables: Dict[str, Any] = {}
    try:
        compiled_config = compile(raw_config, config_path, 'exec')
        exec(compiled_config, variables)
    except Exception:
        print(red(f"❌  Failed to parse {config_path} - config is not valid."))
        logging.error(f"Exception reading {config_path}", exc_info=True)
        return None

    checks = variables.get("checks", [])
    title = variables.get("title", None)
    min_version = variables.get("daktari_version", None)
    return Config(min_version, title, checks)


def check_version_compatibility(config_path: Path, raw_config: str) -> bool:
    match = version_regex.search(raw_config)
    disclaimer = "Specifying daktari_version is recommended to ensure team members have compatible versions installed."
    if match is None:
        print(yellow(f"⚠️  No minimum version found in {config_path}. {disclaimer}"))
        return True

    my_version = version.parse(__version__)
    required_version = version.parse(match.group(1))

    if not isinstance(required_version, version.Version):
        print(red(f"❌  Invalid daktari_version in {config_path}: {required_version}"))
        return False

    logging.debug(f"Doing version check. Mine [{my_version}], required [{required_version}]")
    if required_version > my_version:
        print(
            red(
                f"❌  Installed version of daktari [{my_version}] is "
                f"too old for this project (needs at least {required_version}). "
            )
        )
        print_suggestion_text("pip install daktari -U")
        return False

    return True



from semver import VersionInfo
from typing import Optional


def try_parse_semver(semver_str: Optional[str]) -> VersionInfo:
    try:
        return None if semver_str is None else VersionInfo.parse(semver_str)
    except ValueError:
        return None
